/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <stdlib.h>
#include <assert.h>
#include <math.h>

// define some constants to make referencing in/outs clearer
#define PN_CLOCK 0
#define PN_RESET 1
#define PN_START 2
#define PN_END 3
#define PN_MODE 4
#define PN_COUNT 5
#define PN_NEXTCOUNT 6
#define PN_SEED 7



// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *in1  = buffer_offset + input1_buffer;
	float *in2  = buffer_offset + input2_buffer;
    float *out1 = buffer_offset + output1_buffer;
	float *out2 = buffer_offset + output2_buffer;

    while(--sampleFrames >= 0) 
    {
		float clock = *in1;
		float reset = *in2; 

		if (old_seed != seed)
		{
			FillTable(seed);
			old_seed = seed;
		}

		if( (reset != prev_reset) && (reset > 0.4f) )
		{
			

			switch( mode )
			{
			case 0:
				first_count = 0;
				break;
			case 1:
				first_count = start + 1;
				second_count = start;
				break;
			case 2:
				first_count = end - 1;
				second_count = end;
				break;
			case 3:
				if (direction == 1)	first_count = start;
				else first_count = end;
				break;
			case 4:
				first_count = first_count;
				break;
			}		

		}
		else if( (clock != prev_clock) && (clock > 0.4f) )
		{
			second_count = first_count;
			num_pulses++;

			switch( mode )
			{
			case 0:
				first_count = 0;
				break;
			case 1:
				first_count++;
				if (first_count > end) first_count = start;
				break;
			case 2:
				first_count--;
				if (first_count == start - 1) first_count = end;
				break;
			case 3:
				if( first_count == start ) direction = 1;
				else if( first_count == end ) direction = 0;

				if (direction == 1)	first_count++;
				else first_count--;
				break;
			case 4:
				if( start != end)
				{
					randnum = random_table[num_pulses&511];
					randnum = randnum * (float) (end + 1) - start;

					first_count = (int) randnum + start;
				}
				else first_count = start;
				break;	
			}
			
		}

		prev_reset = reset;
		prev_clock = clock;

		float sc_f = (float) second_count / 10.f;
		float fc_f = (float) first_count / 10.f;

		*out1 = sc_f;
		*out2 = fc_f;

		in1++;
		out1++;
		out2++;
    }
}

void Module::sub_process_static(long buffer_offset, long sampleFrames )
{
	sub_process(buffer_offset, sampleFrames);
	static_count = static_count - sampleFrames;
	if( static_count <= 0 /* && !AreEventsPending()*/ )
	{
		CallHost(seaudioMasterSleepMode);
	}
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	// call the base class
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

	// How to prevent other people using your custom module.
	// query the current registered user's name
	char host_registered_to[100];
	CallHost( seaudioMasterGetRegisteredName, 0,0, host_registered_to );
	bool registered_to_me = strcmp(host_registered_to, "Jeff McClintock") == 0;

	if( !registered_to_me )
	{
		// do something appropriate here if you don't want other people using your module
	}

	switch( mode )
			{
			case 0:
				first_count = 0;
				break;
			case 1:
				first_count = start;
				break;
			case 2:
				first_count = end;
				break;
			case 3:
				first_count = start;
				break;
			case 4:
				first_count = first_count;
				break;
			}		

	prev_clock = -20.f;
	prev_reset = -20.f;
	direction = 1;
	num_pulses = 0;

	random_table = new float [512*sizeof(float)];

	old_seed = -1;
}

Module::~Module()
{
	delete random_table;
}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "OL Counter";								// !!TODO!!

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "OL Counter";						// !!TODO!!

	// Info, may include Author, Web page whatever
	properties->about = "by Oli Larkin";								// !!TODO!!
	return true;
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case 0:
		properties->name				= "Clock";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->datatype_extra		= "5,-5,5,-5";
		properties->default_value		= "-5";
		break;
	case 1:
		properties->name				= "Reset";
		properties->variable_address	= &input2_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->datatype_extra		= "5,-5,5,-5";
		properties->default_value		= "-5";
		break;
	case 2:
		properties->name				= "Start";
		properties->variable_address	= &start;
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		properties->datatype_extra		= "range 1,16";
		properties->default_value		= "1";
		break;
	case 3:
		properties->name				= "End";
		properties->variable_address	= &end;
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		properties->datatype_extra		= "range 1,16";
		properties->default_value		= "16";
		break;
	case 4:
		properties->name				= "Mode";
		properties->variable_address	= &mode;
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		properties->datatype_extra		= "Off=0,Forward=1,Back=2,Fwd+Back=3,Random=4";
		properties->default_value		= "0";
		break;
	case 5:
		properties->name				= "Count";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 6:
		properties->name				= "NextCount";
		properties->variable_address	= &output2_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 7:
		properties->name				= "Random Seed";
		properties->variable_address	= &seed;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_INT;
		properties->default_value		= "1";
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
	state_type out_stat = ST_RUN;

	//if ( getPin(PN_MODE)->getValue() == 0 ){
    //	    out_stat = ST_STATIC;
    //}
//	else out_stat = ST_RUN;

	getPin(PN_COUNT)->TransmitStatusChange( SampleClock(), out_stat);
	getPin(PN_NEXTCOUNT)->TransmitStatusChange( SampleClock(), out_stat);
}

void Module::FillTable(int seedvalue)
{
	srand( seedvalue );

	for (int i=0; i<512 ; i++) 
	{
		random_table[i]= (float) rand()/RAND_MAX;
	}

}
