#include "guimodule.h"
#include "windows.h"
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#define PN_SEED 0
#define PN_FILL 1
#define PN_RANDOMISE 2
#define PN_ARRAY 3
#define NUMSLIDERS 16
#define DEFAULT_VALUE 0

GuiModule::GuiModule(seGuiCallback seaudioMaster, void *p_resvd1) : SEGUI_base(seaudioMaster, p_resvd1)
,m_inhibit_update(false)
{
	CallHost(seGuiHostSetWindowType, 0 );
	CallHost(seGuiHostSetWindowSize, 100, 100 );
	CallHost(seGuiHostSetWindowSizeable, 1 );
}

void GuiModule::Initialise(bool loaded_from_file)
{
	SEGUI_base::Initialise(loaded_from_file);

	pos_array = new int [ NUMSLIDERS * sizeof(int) ];

	if(!loaded_from_file)
	{
		getPin(PN_SEED)->setValueInt(5);
	}

	OnValueChanged();
}

GuiModule::~GuiModule()
{
	delete pos_array;
}

void GuiModule::OnGuiPinValueChange(SeGuiPin *p_pin)
{
	switch( p_pin->getIndex() )
	{
		case PN_SEED:
		{
			srand ( getPin(PN_SEED)->getValueInt() );
			break;
		}
		case PN_FILL:
		{
			InvalidateControl();
			break;
		}
		case PN_RANDOMISE:
		{
			if (getPin(PN_RANDOMISE)->getValueInt() == 1)
			{
				char v[300] = "";
				for( int i = 0 ; i < NUMSLIDERS; i++ )
				{
					char cell[20] = "";
					float r = (float) rand()/RAND_MAX;
					int o;
					if(r > 0.5f)
					{
						o = 1;
					}
					else
					{
						o = 0;
					}
					pos_array[i] = o;
					sprintf(cell,"%i,", o);
					strcat(v,cell);
					assert( strlen(v) < 280 );
				}
				
				getPin(PN_ARRAY)->setValueText( v );

				InvalidateControl();
			}
			break;		
		}
		case PN_ARRAY:
		{
			OnValueChanged();
			break;	
		}
	}
	
}

void GuiModule::parseArrayString( const SeSdkString &p_values )	
{
	SeSdkString s(p_values);

	int i = 0;
	while( s.length() > 0 && i < NUMSLIDERS )
	{
		int comma = s.find(",");

		int val = (int) StringToFloat( s.substr(comma-1,1) );

		pos_array[i] = val;

		i++;
		comma++;
		s = s.substr(comma, s.length() - comma);
	}
}

float GuiModule::StringToFloat(const SeSdkString &p_string)
{
	char * temp;	// convert string to float
	return (float) strtod( p_string.data(), &temp );
}

//Mouse
void GuiModule::OnLButtonDown( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	SetCapture(wi);
	
	int ctl_width = wi->width;

	if ((ctl_width % 2) != 0)	
	{
		ctl_width -= 1;	
	}

	int ctl_height = wi->height;
	int mid_y = ctl_height/2;
	int sl_width = ctl_width/NUMSLIDERS;
	int x_pos = point.x;
	int y_pos = point.y;
	x_pos = max(x_pos,0);
	x_pos = min(x_pos,ctl_width);
	y_pos = max(y_pos,0);
	y_pos = min(y_pos,ctl_height);
	
	for(int i = 0; i < NUMSLIDERS; i++)
	{
		if ( (x_pos > i * sl_width) && (x_pos < (i + 1) * sl_width) )
		{
			which_drag = i;
			//pos_array[which_drag] =  (1.f / (float) ctl_height) * y_pos;
			if(pos_array[which_drag] == 0) 
			{
				pos_array[which_drag] = 1;
			}
			else if(pos_array[which_drag] == 1)
			{
				pos_array[which_drag] = 0;
			}
		}
	}

	char v[300] = "";
	for( int j = 0 ; j < NUMSLIDERS; j++ )
	{
		int val = pos_array[j];
		char cell[20] = "";
		sprintf(cell,"%i,",val);
		strcat(v,cell);
		assert( strlen(v) < 280 );
	}

	m_inhibit_update = true; // prevent jitter due to float->text->float conversion
	SetValueS( v ); // should default to current patch
	m_inhibit_update = false;
	InvalidateControl();
}

void GuiModule::OnMouseMove( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	if( !GetCapture(wi) )
		return;
}

void GuiModule::OnLButtonUp( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	if( !GetCapture(wi) )
	return;

	ReleaseCapture(wi); // get mouse moves

	// clear on-screen drag co-ords
	which_drag = -1;
	InvalidateControl();
}

//Paint
void GuiModule::paint( HDC hDC, SEWndInfo *wi )
{
	int ctl_width = wi->width;

	if ((ctl_width % 2) != 0)	
	{
		ctl_width -= 1;	
	}

	int sl_width = ctl_width/NUMSLIDERS;
	int ctl_height = wi->height;

	// Fill in solid background black
	HBRUSH background_brush = CreateSolidBrush( getPin(PN_FILL)->getValueInt() ); 
	//HBRUSH highlight_brush = CreateSolidBrush( RGB(255,0,0) );

	for(int i = 0; i < NUMSLIDERS; i++)
	{
		//int position = (float) ctl_height * pos_array[i];
		//position = max(position,1);
		//position = min(position,ctl_height - 2);

		//int position = pos_array[i] * ctl_height;

		if(pos_array[i] == 1)
		{
			RECT r;
			r.top = 0;
			r.left = i * sl_width;
			r.right = (i + 1) * sl_width;
			r.bottom = ctl_height;
			FillRect( hDC, &r, background_brush );
		}
		
	}

	// cleanup objects
    DeleteObject(background_brush);
	//DeleteObject(highlight_brush);
}

int GuiModule::InvalidateControl(void)
{
	return CallHost(seGuiHostRequestRepaint);
}

SeSdkString GuiModule::DefaultValue()
{
	char v[300] = "";

	for( int i = 0 ; i < NUMSLIDERS; i++ )
	{
		char cell[20] = "0,";
		pos_array[i] = 0;
		strcat(v,cell);
		assert( strlen(v) < 280 );
	}
	
	return v;
}

SeSdkString GuiModule::GetValueS()
{
	return getPin(PN_ARRAY)->getValueText();
}

void GuiModule::SetValueS(SeSdkString p_string)
{
	getPin(PN_ARRAY)->setValueText(p_string);
}

void GuiModule::OnValueChanged()
{
	if( GetValueS().empty() ) // set a default straight line
	{
		SetValueS( DefaultValue() );
		return; // rely on recursion to re-call this routine
	}

	if( !m_inhibit_update )
	{
		parseArrayString( GetValueS() );
		InvalidateControl();
	}
}